package edu.travel.commodity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.adapter.service.order.OrderItemAdapter;
import edu.travel.adapter.service.tenant.TenantAdapter;
import edu.travel.commodity.entity.ShopReview;
import edu.travel.commodity.mapper.ShopReviewMapper;
import edu.travel.commodity.service.ShopReviewService;
import edu.travel.commodity.utils.ObjectUtils;
import edu.travel.commodity.utils.PageUtil;
import edu.travel.commodity.utils.StringUtil;
import edu.travel.commodity.utils.TokenData;
import edu.travel.remote.dto.AddReviewDto;
import edu.travel.remote.dto.GetOrderItemDto;
import edu.travel.remote.dto.ProductSpecDto;
import edu.travel.exception.BaseException;
import edu.travel.remote.feign.mode.vo.tenant.EduTenantVo;
import edu.travel.remote.vo.*;
import edu.travel.rpc.RPCBaseResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;


@Service
public class ShopReviewServiceImpl extends ServiceImpl<ShopReviewMapper, ShopReview> implements ShopReviewService {
    @Autowired
    private TenantAdapter tenantAdapter;
    @Autowired
    private OrderItemAdapter orderItemAdapter;
    @Override
    public Page<ShopReviewVo> getShopReview(ProductSpecDto params) {
        LambdaQueryWrapper<ShopReview> query = Wrappers.<ShopReview>lambdaQuery()
                .eq(ShopReview::getProductId, params.getProductId())
                .orderByDesc(ShopReview::getIsTop).orderByDesc(ShopReview::getCreateTime);

        Page<ShopReview> page = page(new Page<>(params.getCurrentPage(), params.getPageSize()), query);

        Page<ShopReviewVo> pageEntity = PageUtil.toPageEntity(page, ShopReviewVo.class);
        List<ShopReviewVo> records = pageEntity.getRecords();
        if(ObjectUtil.isEmpty(records)){
            return pageEntity;
        }
        Set<String> collect = records.stream().map(ShopReviewVo::getUserId).collect(Collectors.toSet());
        RPCBaseResponse<List<EduTenantVo>> tenantByIds = tenantAdapter.getTenantByIds(collect);
        //继续抛出
        if(tenantByIds.getCode()!=200){
            throw new BaseException(tenantByIds.getCode(), tenantByIds.getMsg());
        }
        List<EduTenantVo> data = tenantByIds.getData();
        HashMap<String, EduTenantVo> users = new HashMap<>();
        for (EduTenantVo datum : data) {
            users.put(datum.getId(), datum);
        }
        for (ShopReviewVo record : records) {
            List<String> list = StringUtil.toList(record.getImage());
            record.setImageUrls(list);
            if(!ObjectUtil.isEmpty(record.getSkuDetails())){
                String jsonPrettyStr = JSONUtil.toJsonPrettyStr(record.getSkuDetails());
                List<SpecOneVo> list1 = JSONUtil.toList(jsonPrettyStr, SpecOneVo.class);
                record.setSpecOneVo(list1);
            }
            EduTenantVo eduTenantVo = users.get(record.getUserId());
            record.setEduTenantVo(eduTenantVo);
        }
        return pageEntity;
    }

    @Override
    public String addReview(AddReviewDto params) {
        String userId = TokenData.getUserId();
        LambdaQueryWrapper<ShopReview> query = Wrappers.<ShopReview>lambdaQuery().eq(ShopReview::getUserId, userId)
                .eq(ShopReview::getOrderId, params.getOrderId())
                .eq(ShopReview::getProductId, params.getProductId());
        List<ShopReview> list = list(query);
        if(!ObjectUtil.isEmpty(list)){
            return "您已经评价过了";
        }
        ShopReview bean = BeanUtil.toBean(params, ShopReview.class);
        bean.setUserId(Long.valueOf(userId));
        String image = StringUtil.toListString(params.getImage());
        bean.setImage(image);
        //获取商品sku快照 SpecOneVo JSON
        GetOrderItemDto getOrderItemDto = new GetOrderItemDto();
        getOrderItemDto.setOrderId(params.getOrderId());
        getOrderItemDto.setSkuId(params.getSkuId());
        getOrderItemDto.setProductId(params.getProductId());
        RPCBaseResponse<OrderItemVo> result = orderItemAdapter.getOrderItemVo(getOrderItemDto);
        if(result.getCode()!=200){
            throw new BaseException(result.getCode(), result.getMsg());
        }
        OrderItemVo data = result.getData();
        if(data == null){
            return "订单信息有误";
        }
        String snapshot = data.getSnapshot();
        if(!ObjectUtil.isEmpty(snapshot)){
            //订单快照对象
            ShopSnapshotVo bean1 = JSONUtil.toBean(snapshot, ShopSnapshotVo.class);
            List<Object> snapshotSpec = bean1.getSnapshotSpec();
            List<SpecValueVo> snapshot1 = ObjectUtils.castList(snapshotSpec, SpecValueVo.class);
            String jsonStr = JSONUtil.toJsonStr(snapshot1);
            bean.setSkuDetails(jsonStr);
        }
        bean.setUpdateUserId(userId);
        save(bean);
        return "评价成功";
    }
}
