package edu.travel.order.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import edu.travel.order.entity.ShopOrder;
import edu.travel.order.service.ShopOrderService;
import edu.travel.remote.dto.AddOrderDto;
import edu.travel.remote.dto.GetOrderItemInfoByOrderNumDto;
import edu.travel.remote.dto.ShowOrderInfoDto;
import edu.travel.remote.order.ShopOrderRemoteController;
import edu.travel.remote.vo.OrderItemVo;
import edu.travel.remote.vo.ShowOrderInfoVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.web.BaseController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import static edu.travel.rpc.RPCBaseResponse.success;


@RestController
@RequestMapping("/shopOrder")
public class ShopOrderController extends BaseController<ShopOrder> implements ShopOrderRemoteController {
    @Autowired
    private ShopOrderService shopOrderService;

    /**
     *  状态查询我的订单
     * @param params
     * @return {@link RPCBaseResponse }<{@link Page }<{@link ShowOrderInfoVo }>>
     */
    @GetMapping("/showOrderInfo")
    public RPCBaseResponse<Page<ShowOrderInfoVo>> showOrderInfo(ShowOrderInfoDto params){
        return success(shopOrderService.showOrderInfo(params));
    }
    /**
     *  下单生成订单
     * @return {@link RPCBaseResponse }<{@link Void }>
     */
    @Override
    @PostMapping("/addShopOrder")
    public RPCBaseResponse<Void> addShopOrder(@RequestBody AddOrderDto params) {
        shopOrderService.addShopOrder(params);
        return success();
    }
    /**
     *  查询订单下的商品快照
     * @param params 订单号
     * @return {@link RPCBaseResponse }<{@link String }>
     */

    @GetMapping("/getOrderItemInfo")
    public RPCBaseResponse<Page<OrderItemVo>> getOrderItemInfo(GetOrderItemInfoByOrderNumDto params){
        return RPCBaseResponse.success(shopOrderService.getOrderItemInfo(params));
    }


}
