package edu.travel.tenant.web;

import edu.travel.remote.feign.mode.dto.tenant.BannerDto;
import edu.travel.remote.feign.mode.romote.ShopBannerRemoteController;
import edu.travel.remote.feign.mode.vo.banner.BannerVo;
import edu.travel.resp.BaseResponse;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.tenant.entity.ShopBanner;
import edu.travel.tenant.service.ShopBannerService;
import edu.travel.web.BaseController;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

import static edu.travel.rpc.RPCBaseResponse.success;

/**
* 轮播图(shop_banner)表控制层
*
* @author xxxxx
*/
@RestController
@RequestMapping("/shopBanner")
public class ShopBannerController extends BaseController<ShopBanner> implements ShopBannerRemoteController {
/**
* 服务对象
*/
    @Autowired
    private ShopBannerService shopBannerService;


    /**
     *  获取轮播图
     * @param banner
     * @return {@link BaseResponse }<{@link List }<{@link BannerVo }>>
     */
    @GetMapping("/getBanner")
    public RPCBaseResponse<List<BannerVo>> getBanner(BannerDto banner){
        return success(shopBannerService.getBanner(banner));
    }

    /**
     * 上传banner图片
     * @param
     * @return
     */
    @GetMapping("/uploadBannerImage")
    public ResponseEntity<RPCBaseResponse<String>> uploadBannerImage(@RequestBody MultipartFile file){
        return ResponseEntity.ok(shopBannerService.uploadBannerImage(file));
    }

    @Override
    @GetMapping("/getFormId")
    public RPCBaseResponse<BannerVo> getFormId(String id) {
        RPCBaseResponse<ShopBanner> shopBannerRPCBaseResponse = super.getId(id);
        RPCBaseResponse<BannerVo> shopBannerVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopBannerRPCBaseResponse, shopBannerVoRPCBaseResponse);
        return shopBannerVoRPCBaseResponse;
    }

    @Override
    @GetMapping("/updateFormTarget")
    public RPCBaseResponse<BannerVo> updateTargetFormId(@RequestBody BannerDto entity) {
        ShopBanner shopBanner = new ShopBanner();
        BeanUtils.copyProperties(entity, shopBanner);
        RPCBaseResponse<ShopBanner> shopBannerRPCBaseResponse = super.updateTargetById(shopBanner);
        RPCBaseResponse<BannerVo> shopBannerVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopBannerRPCBaseResponse, shopBannerVoRPCBaseResponse);
        return shopBannerVoRPCBaseResponse;
    }

    @Override
    @GetMapping("/saveFormTarget")
    public RPCBaseResponse<BannerVo> saveFormTarget(BannerDto entity) {
        ShopBanner shopBanner = new ShopBanner();
        BeanUtils.copyProperties(entity, shopBanner);
        RPCBaseResponse<ShopBanner> shopBannerRPCBaseResponse = super.saveTarget(shopBanner);
        RPCBaseResponse<BannerVo> shopBannerVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopBannerRPCBaseResponse, shopBannerVoRPCBaseResponse);
        return shopBannerVoRPCBaseResponse;
    }


    @Override
    @GetMapping("/deleteFormTarget")
    public RPCBaseResponse<BannerVo> deleteTargetFormId(List<String> ids) {
        RPCBaseResponse<ShopBanner> shopBannerRPCBaseResponse = super.deleteTargetById(ids);
        RPCBaseResponse<BannerVo> shopBannerVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopBannerRPCBaseResponse, shopBannerVoRPCBaseResponse);
        return shopBannerVoRPCBaseResponse;
    }

    @Override
    @GetMapping("/getAllForm")
    public RPCBaseResponse<List<BannerVo>> getAllForm() {
        RPCBaseResponse<List<ShopBanner>> countryRPCBaseResponse = super.listAll();
        RPCBaseResponse<List<BannerVo>> baseCountryVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(countryRPCBaseResponse, baseCountryVoRPCBaseResponse);
        return baseCountryVoRPCBaseResponse;
    }

}
